#!/usr/bin/env python
# coding: utf-8

# In[15]:


import tkinter as tk
import tkinter.messagebox as msg
from tkinter.filedialog import askopenfilename, asksaveasfilename
from PIL import Image, ImageTk
import os
import json

# Layout constants.
MARGIN = 5
WINDOW_WID = 350
WINDOW_HGT = 200

class MruApp:
    # Create and manage the tkinter interface.
    def __init__(self):
        # Load the MRU list.
        self.max_mru_length = 4
        self.load_mru_list()

        # Make the main interface.
        self.window = tk.Tk()
        self.window.title('MruApp')
        self.window.protocol('WM_DELETE_WINDOW', self.kill_callback)
        self.window.geometry(f'{WINDOW_WID}x{WINDOW_HGT}')

        # Build the menus.
        self.build_menus()

        # Build the rest of the UI here...

        # Display the window.
        self.window.focus_force()
        self.window.mainloop()

    def load_mru_list(self):
        '''Load the MRU list.'''
        self.mru_list = []
        try:
            # Load the MRU list JSON file.
            with open('mru_list.json', 'r') as f:
                file_dict = json.load(f)

            # Pull the file names out of the loaded data.
            for i in range(self.max_mru_length):
                key = f'MRU_{i}'
                if key in file_dict:
                    self.mru_list.append(file_dict[key])
        except:
            pass

    def save_mru_list(self):
        '''Save the MRU list.'''
        # Build a dictionary holding the MRU file names.
        file_dict = {}
        for i in range(len(self.mru_list)):
            key = f'MRU_{i}'
            file_dict[key] = self.mru_list[i]

        # Save the JSON file.
        with open('mru_list.json', 'w') as f:
            json.dump(file_dict, f)

    def build_menus(self):
        '''Build the menu bar.'''
        menu_bar = tk.Menu(self.window)
        self.window.config(menu=menu_bar)

        self.build_file_menu(menu_bar)

    def build_file_menu(self, menu_bar):
        '''Build the File menu.'''
        file_menu = tk.Menu(menu_bar, tearoff=False)
        menu_bar.add_cascade(label='File', menu=file_menu)

        # Note: You must save the photo image in a persistent
        # variable so tkinter can redraw it when needed.

        # Note: You need to include the compound parameter
        # if you want to use both images and text. That value
        # tells where to put the image relative to the text.

        # Open
        self.open_photo_image = ImageTk.PhotoImage(file='open.png')
        file_menu.add_command(label='Open...', accelerator='Ctrl+O', compound='left',
            command=self.mnu_open, image=self.open_photo_image)
        self.window.bind('<Control-o>', lambda event: self.mnu_open())

        # Save As
        self.save_as_photo_image = ImageTk.PhotoImage(file='save_as.png')
        file_menu.add_command(label='Save As...', accelerator='Ctrl+S', compound='left',
            command=self.mnu_save_as, image=self.save_as_photo_image)
        self.window.bind('<Control-s>', lambda event: self.mnu_save_as())

        file_menu.add_separator()

        # MRU menu
        self.file_menu = file_menu # Save so we can enable/disable the Recent Files command.
        self.mru_menu = tk.Menu(file_menu, tearoff=False)
        self.mru_list_photo_image = ImageTk.PhotoImage(file='mru_list.png')
        file_menu.add_cascade(label='Recent Files', menu=self.mru_menu,
            compound='left', image=self.mru_list_photo_image)

        # Make the initial MRU list.
        self.document_photo_image = ImageTk.PhotoImage(file='document.png')
        self.build_mru_list()

        # Exit
        self.exit_photo_image = ImageTk.PhotoImage(file='exit.png')
        file_menu.add_command(label='Exit', compound='left',
            command=self.mnu_exit, image=self.exit_photo_image)

    def build_mru_list(self):
        '''Add files to the MRU menu.'''
        # Delete existing items.
        self.mru_menu.delete(0, tk.END)

        # Add the files.
        # Note how the lambda function renames the variable.
        # This forces it to bind the value when this code is
        # executed rather than trying to use the value of
        # filename when the lambda executes.
        for filename in self.mru_list:
            label = os.path.basename(filename)
            self.mru_menu.add_command(label=label, compound='left',
                command=lambda bound_name=filename: self.mru_file_selected(bound_name),
                image=self.document_photo_image)

        # Enable the Recent Files command if there are no recent files.
        if len(self.mru_list) > 0:
            state = 'normal'
        else:
            state = 'disabled'
        self.file_menu.entryconfigure('Recent Files', state=state)

        # Save the MRU list.
        self.save_mru_list()

    def mru_file_selected(self, filename):
        self.open_file(filename)

    def mnu_open(self):
        '''Open a file.'''
        filetypes = [
            ('Text files', '*.txt'),
            ('Python Files', '*.py *.ipynb'),
            ('All Files', '*,*')]
        filename = askopenfilename(filetypes=filetypes)
        if len(filename) > 0:
            # Open the file.
            self.open_file(filename)

    def open_file(self, filename):
        '''Open a file and update the MRU list.'''
        # Add your code to open the file here.
        msg.showinfo('Open File', filename)

        # If the open succeeds, update the MRU list.
        # Otherwise remove the file name from the MRU list.
        open_succeeded = True
        if open_succeeded:
            self.mru_add_file(filename)
        else:
            self.mru_remove_file(filename)

    def mru_add_file(self, filename):
        '''Add the file name to the MRU list.'''
        # Move the file name to the top of the list.
        if filename in self.mru_list:
            self.mru_list.remove(filename)
        self.mru_list.insert(0, filename)

        # limit the number of names on the list.
        self.mru_list = self.mru_list[:self.max_mru_length]

        # Rebuild the MRU menu.
        self.build_mru_list()

    def mru_remove_file(self, filename):
        '''Remove the file name from the MRU list.'''
        if filename in self.mru_list:
            self.mru_list.remove(filename)

        # Rebuild the MRU menu.
        self.build_mru_list()

    def mnu_save_as(self):
        '''Save a file.'''
        filetypes = [
            ('Text files', '*.txt'),
            ('Python Files', '*.py *.ipynb'),
            ('All Files', '*,*')]
        filename = asksaveasfilename(filetypes=filetypes)
        if len(filename) > 0:
            # Save the file.
            msg.showinfo('Save File', filename)

            # If the save succeeds, update the MRU list.
            save_succeeded = True
            if save_succeeded:
                self.mru_add_file(filename)

    def mnu_exit(self):
        print('Exit')
        self.kill_callback()

    def kill_callback(self):
        self.window.destroy()


# In[16]:


MruApp()
print('Done')


# In[ ]:




